@@ -138,6 +138,7 @@ if applicable the created ArticleID.
                 ForceNotificationToUserID       => [1, 2, 3]                   # optional
                 ExcludeNotificationToUserID     => [1, 2, 3]                   # optional
                 ExcludeMuteNotificationToUserID => [1, 2, 3]                   # optional
+                AppendSignatureToBody           => 1,                          # optional, defaults to 1
                 Attachment => [
                     {
                         Content     => 'content'                                 # base64 encoded
@@ -607,7 +608,7 @@ sub Run {
     for my $AttachmentItem (@AttachmentList) {
         if ( !IsHashRefWithData($AttachmentItem) ) {
             return {
-                ErrorCode => 'TicketUpdate.InvalidParameter',
+                ErrorCode    => 'TicketUpdate.InvalidParameter',
                 ErrorMessage =>
                     "TicketUpdate: Ticket->Attachment parameter is invalid!",
             };
@@ -634,6 +635,7 @@ sub Run {
         TicketDynamicFields => $DynamicFieldList,
         AttachmentList      => \@AttachmentList,
         UserID              => $UserID,
+        PermissionUserID    => $PermissionUserID,
         UserType            => $UserType,
     );
 }
@@ -674,7 +676,7 @@ sub _CheckTicket {
         )
     {
         return {
-            ErrorCode => 'TicketUpdate.InvalidParameter',
+            ErrorCode    => 'TicketUpdate.InvalidParameter',
             ErrorMessage =>
                 "TicketUpdate: Ticket->CustomerUser parameter is invalid!",
         };
@@ -706,7 +708,7 @@ sub _CheckTicket {
     if ( $Ticket->{TypeID} || $Ticket->{Type} ) {
         if ( !$Self->ValidateType( %{$Ticket} ) ) {
             return {
-                ErrorCode => 'TicketUpdate.InvalidParameter',
+                ErrorCode    => 'TicketUpdate.InvalidParameter',
                 ErrorMessage =>
                     "TicketUpdate: Ticket->TypeID or Ticket->Type parameter is invalid!",
             };
@@ -729,7 +731,7 @@ sub _CheckTicket {
             )
         {
             return {
-                ErrorCode => 'TicketUpdate.InvalidParameter',
+                ErrorCode    => 'TicketUpdate.InvalidParameter',
                 ErrorMessage =>
                     "TicketUpdate: Ticket->ServiceID or Ticket->Service parameter is invalid!",
             };
@@ -755,7 +757,7 @@ sub _CheckTicket {
             )
         {
             return {
-                ErrorCode => 'TicketUpdate.InvalidParameter',
+                ErrorCode    => 'TicketUpdate.InvalidParameter',
                 ErrorMessage =>
                     "TicketUpdate: Ticket->SLAID or Ticket->SLA parameter is invalid!",
             };
@@ -788,7 +790,7 @@ sub _CheckTicket {
     if ( $Ticket->{OwnerID} || $Ticket->{Owner} ) {
         if ( !$Self->ValidateOwner( %{$Ticket} ) ) {
             return {
-                ErrorCode => 'TicketUpdate.InvalidParameter',
+                ErrorCode    => 'TicketUpdate.InvalidParameter',
                 ErrorMessage =>
                     "TicketUpdate: Ticket->OwnerID or Ticket->Owner parameter is invalid!",
             };
@@ -917,7 +919,7 @@ sub _CheckArticle {
         )
     {
         return {
-            ErrorCode => 'TicketCreate.InvalidParameter',
+            ErrorCode    => 'TicketCreate.InvalidParameter',
             ErrorMessage =>
                 "TicketCreate: Article->To parameter must be a valid email address when Article->ArticleSend is set!",
         };
@@ -1144,7 +1146,7 @@ sub _ValidateDynamicFields {
     for my $DynamicFieldItem (@$DynamicFieldList) {
         if ( !IsHashRefWithData($DynamicFieldItem) ) {
             return {
-                ErrorCode => 'TicketUpdate.InvalidParameter',
+                ErrorCode    => 'TicketUpdate.InvalidParameter',
                 ErrorMessage =>
                     "TicketUpdate: Ticket->DynamicField parameter is invalid!",
             };
@@ -1206,7 +1208,7 @@ sub _CheckDynamicField {
         )
     {
         return {
-            ErrorCode => 'TicketUpdate.MissingParameter',
+            ErrorCode    => 'TicketUpdate.MissingParameter',
             ErrorMessage =>
                 "TicketUpdate: To create an article DynamicField an article is required!",
         };
@@ -1352,9 +1354,13 @@ sub _CheckUpdatePermissions {
     # get ticket object
     my $TicketObject = $Kernel::OM->Get('Kernel::System::Ticket');
 
+    my $TicketPermissionFunctionName = $Param{UserType} eq 'Customer'
+        ? 'TicketCustomerPermission'
+        : 'TicketPermission';
+
     # check Article permissions
     if ( IsHashRefWithData($Article) ) {
-        my $Access = $TicketObject->TicketPermission(
+        my $Access = $TicketObject->$TicketPermissionFunctionName(
             Type     => 'note',
             TicketID => $TicketID,
             UserID   => $Param{UserID},
@@ -1369,7 +1375,7 @@ sub _CheckUpdatePermissions {
 
     # check dynamic field permissions
     if ( IsArrayRefWithData($DynamicFieldList) ) {
-        my $Access = $TicketObject->TicketPermission(
+        my $Access = $TicketObject->$TicketPermissionFunctionName(
             Type     => 'rw',
             TicketID => $TicketID,
             UserID   => $Param{UserID},
@@ -1384,7 +1390,7 @@ sub _CheckUpdatePermissions {
 
     # check queue permissions
     if ( $Ticket->{Queue} || $Ticket->{QueueID} ) {
-        my $Access = $TicketObject->TicketPermission(
+        my $Access = $TicketObject->$TicketPermissionFunctionName(
             Type     => 'move',
             TicketID => $TicketID,
             UserID   => $Param{UserID},
@@ -1399,7 +1405,7 @@ sub _CheckUpdatePermissions {
 
     # check owner permissions
     if ( $Ticket->{Owner} || $Ticket->{OwnerID} ) {
-        my $Access = $TicketObject->TicketPermission(
+        my $Access = $TicketObject->$TicketPermissionFunctionName(
             Type     => 'owner',
             TicketID => $TicketID,
             UserID   => $Param{UserID},
@@ -1414,7 +1420,7 @@ sub _CheckUpdatePermissions {
 
     # check responsible permissions
     if ( $Ticket->{Responsible} || $Ticket->{ResponsibleID} ) {
-        my $Access = $TicketObject->TicketPermission(
+        my $Access = $TicketObject->$TicketPermissionFunctionName(
             Type     => 'responsible',
             TicketID => $TicketID,
             UserID   => $Param{UserID},
@@ -1429,7 +1435,7 @@ sub _CheckUpdatePermissions {
 
     # check priority permissions
     if ( $Ticket->{Priority} || $Ticket->{PriorityID} ) {
-        my $Access = $TicketObject->TicketPermission(
+        my $Access = $TicketObject->$TicketPermissionFunctionName(
             Type     => 'priority',
             TicketID => $TicketID,
             UserID   => $Param{UserID},
@@ -1468,7 +1474,7 @@ sub _CheckUpdatePermissions {
         my $Access = 1;
 
         if ( $StateData{TypeName} =~ /^close/i ) {
-            $Access = $TicketObject->TicketPermission(
+            $Access = $TicketObject->$TicketPermissionFunctionName(
                 Type     => 'close',
                 TicketID => $TicketID,
                 UserID   => $Param{UserID},
@@ -1477,7 +1483,7 @@ sub _CheckUpdatePermissions {
 
         # set pending time
         elsif ( $StateData{TypeName} =~ /^pending/i ) {
-            $Access = $TicketObject->TicketPermission(
+            $Access = $TicketObject->$TicketPermissionFunctionName(
                 Type     => 'close',
                 TicketID => $TicketID,
                 UserID   => $Param{UserID},
@@ -1501,13 +1507,14 @@ sub _CheckUpdatePermissions {
 updates a ticket and creates an article and sets dynamic fields and attachments if specified.
 
     my $Response = $OperationObject->_TicketUpdate(
-        TicketID     => 123,
-        Ticket       => $Ticket,                  # all ticket parameters
-        Articles     => @Articles,                # all article parameters, optionally with dynamic fields
-        DynamicField => $DynamicField,            # all ticket dynamic field parameters
-        Attachment   => $Attachment,              # all attachment parameters
-        UserID       => 123,
-        UserType     => 'Agent'                   # || 'Customer
+        TicketID         => 123,
+        Ticket           => $Ticket,                  # all ticket parameters
+        Articles         => @Articles,                # all article parameters, optionally with dynamic fields
+        DynamicField     => $DynamicField,            # all ticket dynamic field parameters
+        Attachment       => $Attachment,              # all attachment parameters
+        UserID           => 123,
+        PermissionUserID => 201,                      # User for which permissions will be checked
+        UserType         => 'Agent'                   # || 'Customer
     );
 
     returns:
@@ -1538,7 +1545,10 @@ sub _TicketUpdate {
     my $TicketDynamicFields = $Param{TicketDynamicFields};
     my $AttachmentList      = $Param{AttachmentList};
 
-    my $Access = $Self->_CheckUpdatePermissions(%Param);
+    my $Access = $Self->_CheckUpdatePermissions(
+        %Param,
+        UserID => $Param{PermissionUserID},
+    );
 
     # if no permissions return error
     if ( !$Access->{Success} ) {
@@ -1579,7 +1589,7 @@ sub _TicketUpdate {
         );
         if ( !$Success ) {
             return {
-                Success => 0,
+                Success      => 0,
                 Errormessage =>
                     'Ticket title could not be updated, please contact system administrator!',
             };
@@ -1611,7 +1621,7 @@ sub _TicketUpdate {
 
         if ( !$Success ) {
             return {
-                Success => 0,
+                Success      => 0,
                 ErrorMessage =>
                     'Ticket queue could not be updated, please contact system administrator!',
             };
@@ -1643,7 +1653,7 @@ sub _TicketUpdate {
 
         if ( !$Success ) {
             return {
-                Success => 0,
+                Success      => 0,
                 Errormessage =>
                     'Ticket lock could not be updated, please contact system administrator!',
             };
@@ -1676,7 +1686,7 @@ sub _TicketUpdate {
 
         if ( !$Success ) {
             return {
-                Success => 0,
+                Success      => 0,
                 Errormessage =>
                     'Ticket type could not be updated, please contact system administrator!',
             };
@@ -1731,7 +1741,7 @@ sub _TicketUpdate {
 
                 if ( !$Success ) {
                     return {
-                        Success => 0,
+                        Success      => 0,
                         Errormessage =>
                             'Ticket pendig time could not be updated, please contact system'
                             . ' administrator!',
@@ -1770,7 +1780,7 @@ sub _TicketUpdate {
 
         if ( !$Success ) {
             return {
-                Success => 0,
+                Success      => 0,
                 Errormessage =>
                     'Ticket state could not be updated, please contact system administrator!',
             };
@@ -1788,7 +1798,7 @@ sub _TicketUpdate {
             if (
                 !$Self->ValidateSLA(
                     SLAID     => $TicketData{SLAID},
-                    Service   => $Ticket->{Service} || '',
+                    Service   => $Ticket->{Service}   || '',
                     ServiceID => $Ticket->{ServiceID} || '',
                 )
                 )
@@ -1836,7 +1846,7 @@ sub _TicketUpdate {
 
         if ( !$Success ) {
             return {
-                Success => 0,
+                Success      => 0,
                 Errormessage =>
                     'Ticket service could not be updated, please contact system administrator!',
             };
@@ -1878,7 +1888,7 @@ sub _TicketUpdate {
 
         if ( !$Success ) {
             return {
-                Success => 0,
+                Success      => 0,
                 Errormessage =>
                     'Ticket SLA could not be updated, please contact system administrator!',
             };
@@ -1922,7 +1932,7 @@ sub _TicketUpdate {
 
         if ( !$Success ) {
             return {
-                Success => 0,
+                Success      => 0,
                 Errormessage =>
                     'Ticket customer user could not be updated, please contact system administrator!',
             };
@@ -1955,7 +1965,7 @@ sub _TicketUpdate {
 
         if ( !$Success ) {
             return {
-                Success => 0,
+                Success      => 0,
                 Errormessage =>
                     'Ticket priority could not be updated, please contact system administrator!',
             };
@@ -1992,7 +2002,7 @@ sub _TicketUpdate {
 
         if ( !$Success ) {
             return {
-                Success => 0,
+                Success      => 0,
                 Errormessage =>
                     'Ticket owner could not be updated, please contact system administrator!',
             };
@@ -2032,7 +2042,7 @@ sub _TicketUpdate {
 
         if ( !$Success ) {
             return {
-                Success => 0,
+                Success      => 0,
                 Errormessage =>
                     'Ticket responsible could not be updated, please contact system administrator!',
             };
@@ -2157,7 +2167,7 @@ sub _TicketUpdate {
 
                 if ( !$Subject ) {
                     return {
-                        Success => 0,
+                        Success      => 0,
                         ErrorMessage =>
                             'The subject for the e-mail could not be generated. Please contact the system administrator'
                     };
@@ -2167,30 +2177,47 @@ sub _TicketUpdate {
                 # Template generator implicitly takes Frontend::RichText into account.
                 # Temporarily enable/disable RichText setting according to content type of article,
                 # so that body and signature both are plain text or HTML.
+                # Default is adding the signature (previous standard behavior).
                 #
-                my $ConfigObject = $Kernel::OM->Get('Kernel::Config');
+                if ( $Article->{AppendSignatureToBody} // 1 ) {
+                    my $ConfigObject = $Kernel::OM->Get('Kernel::Config');
 
-                my $OriginalRichTextSetting = $ConfigObject->Get('Frontend::RichText');
+                    my $OriginalRichTextSetting = $ConfigObject->Get('Frontend::RichText');
 
-                $ConfigObject->{'Frontend::RichText'} = 0 if $ArticleIsPlainText;
-                $ConfigObject->{'Frontend::RichText'} = 1 if $ArticleIsHTML;
+                    $ConfigObject->{'Frontend::RichText'} = 0 if $ArticleIsPlainText;
+                    $ConfigObject->{'Frontend::RichText'} = 1 if $ArticleIsHTML;
 
-                my $Signature = $Kernel::OM->Get('Kernel::System::TemplateGenerator')->Signature(
-                    TicketID => $TicketID,
-                    UserID   => $Param{UserID},
-                    Data     => $Article,
-                );
+                    # To make sure to have correct richtext setting in template generator
+                    $Kernel::OM->ObjectsDiscard(
+                        Objects => [
+                            'Kernel::System::TemplateGenerator',
+                        ],
+                    );
 
-                # Restore original RichText setting.
-                $ConfigObject->{'Frontend::RichText'} = $OriginalRichTextSetting;
+                    my $Signature = $Kernel::OM->Get('Kernel::System::TemplateGenerator')->Signature(
+                        TicketID => $TicketID,
+                        UserID   => $Param{UserID},
+                        Data     => $Article,
+                    );
 
-                if ($Signature) {
-                    $Article->{Body} = $Article->{Body} . $Signature;
+                    # Restore original RichText setting.
+                    $ConfigObject->{'Frontend::RichText'} = $OriginalRichTextSetting;
 
-                    if ($ArticleIsHTML) {
-                        $PlainBody = $Kernel::OM->Get('Kernel::System::HTMLUtils')->ToAscii(
-                            String => $Article->{Body},
-                        );
+                    # To make sure to have correct richtext setting in template generator
+                    $Kernel::OM->ObjectsDiscard(
+                        Objects => [
+                            'Kernel::System::TemplateGenerator',
+                        ],
+                    );
+
+                    if ($Signature) {
+                        $Article->{Body} = $Article->{Body} . $Signature;
+
+                        if ($ArticleIsHTML) {
+                            $PlainBody = $Kernel::OM->Get('Kernel::System::HTMLUtils')->ToAscii(
+                                String => $Article->{Body},
+                            );
+                        }
                     }
                 }
             }
@@ -2238,7 +2265,7 @@ sub _TicketUpdate {
                 NoAgentNotify        => $Article->{NoAgentNotify} || 0,
                 TicketID             => $TicketID,
                 SenderTypeID         => $Article->{SenderTypeID} || '',
-                SenderType           => $Article->{SenderType} || '',
+                SenderType           => $Article->{SenderType}   || '',
                 IsVisibleForCustomer => $Article->{IsVisibleForCustomer},
                 From                 => $From,
                 To                   => $To,
@@ -2246,8 +2273,8 @@ sub _TicketUpdate {
                 Bcc                  => $Bcc,
                 Subject              => $Subject,
                 Body                 => $Article->{Body},
-                MimeType             => $MimeType || '',
-                Charset              => $Charset || '',
+                MimeType             => $MimeType               || '',
+                Charset              => $Charset                || '',
                 ContentType          => $Article->{ContentType} || '',
                 UserID               => $Param{UserID},
                 HistoryType          => $Article->{HistoryType},
@@ -2318,7 +2345,7 @@ sub _TicketUpdate {
 
             if ( !$ArticleID ) {
                 return {
-                    Success => 0,
+                    Success      => 0,
                     ErrorMessage =>
                         'Article could not be created, please contact the system administrator'
                 };
@@ -2326,8 +2353,10 @@ sub _TicketUpdate {
 
             if ( $Article->{DynamicField} ) {
                 my $ArticleDynamicFieldList = _MakeArrayRef( $Article->{DynamicField} );
-                if ( my $Result
-                    = $Self->_SetDynamicFields( $ArticleDynamicFieldList, $TicketID, $ArticleID, $Param{UserID} ) )
+                if (
+                    my $Result
+                    = $Self->_SetDynamicFields( $ArticleDynamicFieldList, $TicketID, $ArticleID, $Param{UserID} )
+                    )
                 {
                     return $Result;
                 }
