@@ -1917,6 +1917,8 @@ sub _Mask {
         $Param{OptionCustomerUserAddressBook} = 1;
     }
 
+    my $PreviewContentTypes = $ConfigObject->Get('Attachment')->{PreviewContentTypes} || {};
+
     # show attachments
     ATTACHMENT:
     for my $Attachment ( @{ $Param{Attachments} } ) {
@@ -1932,6 +1934,12 @@ sub _Mask {
             }
         }
 
+        # Add preview flag if content type is in the preview content types list.
+        # This is used to determine if the attachment can be previewed in the UI.
+        if ( $Attachment->{ContentType} && $PreviewContentTypes->{ $Attachment->{ContentType} } ) {
+            $Attachment->{Preview} = 1;
+        }
+
         push @{ $Param{AttachmentList} }, $Attachment;
     }
 
