@@ -400,6 +400,7 @@ sub Run {
             SLAID        => $Ticket{SLAID},
             TicketNumber => $Ticket{TicketNumber},
             Title        => $Ticket{Title},
+            State        => $Ticket{State},
             NextStates   => $Self->_GetNextStates(
                 %GetParam,
             ),
@@ -787,6 +788,7 @@ sub Run {
                 QueueID      => $Ticket{QueueID},
                 SLAID        => $Ticket{SLAID},
                 Title        => $Ticket{Title},
+                State        => $Ticket{State},
                 NextStates   => $Self->_GetNextStates(
                     %GetParam,
                 ),
@@ -1310,9 +1312,13 @@ sub _MaskPhone {
     if ( $Param{NextStateID} ) {
         $Selected{SelectedID} = $Param{NextStateID};
     }
-    elsif ( $Config->{State} ) {
-        $Selected{SelectedValue} = $Config->{State};
+    elsif ( $Config->{StateDefault} ) {
+        $Selected{SelectedValue} = $Config->{StateDefault};
     }
+    else {
+        $Selected{SelectedValue} = $Param{State};
+    }
+
     $Param{NextStatesStrg} = $LayoutObject->BuildSelection(
         Data         => $Param{NextStates},
         Name         => 'NextStateID',
@@ -1444,6 +1450,8 @@ sub _MaskPhone {
         );
     }
 
+    my $PreviewContentTypes = $ConfigObject->Get('Attachment')->{PreviewContentTypes} || {};
+
     # show attachments
     ATTACHMENT:
     for my $Attachment ( @{ $Param{Attachments} } ) {
@@ -1457,6 +1465,12 @@ sub _MaskPhone {
             next ATTACHMENT;
         }
 
+        # Add preview flag if content type is in the preview content types list.
+        # This is used to determine if the attachment can be previewed in the UI.
+        if ( $Attachment->{ContentType} && $PreviewContentTypes->{ $Attachment->{ContentType} } ) {
+            $Attachment->{Preview} = 1;
+        }
+
         push @{ $Param{AttachmentList} }, $Attachment;
     }
 
