@@ -1238,6 +1238,8 @@ sub _MaskNew {
         );
     }
 
+    my $PreviewContentTypes = $ConfigObject->Get('Attachment')->{PreviewContentTypes} || {};
+
     # show attachments
     ATTACHMENT:
     for my $Attachment ( @{ $Param{Attachments} } ) {
@@ -1251,6 +1253,12 @@ sub _MaskNew {
             next ATTACHMENT;
         }
 
+        # Add preview flag if content type is in the preview content types list.
+        # This is used to determine if the attachment can be previewed in the UI.
+        if ( $Attachment->{ContentType} && $PreviewContentTypes->{ $Attachment->{ContentType} } ) {
+            $Attachment->{Preview} = 1;
+        }
+
         push @{ $Param{AttachmentList} }, $Attachment;
     }
 
