@@ -10,8 +10,11 @@ package Kernel::Output::CSS::Base;
 
 use strict;
 use warnings;
+use utf8;
 
-use Kernel::System::VariableCheck qw(:all);
+=head1 NAME
+
+Kernel::Output::CSS::Base - output css module base class
 
 =head2 new()
 
