@@ -133,7 +133,6 @@ Events:
 sub AppointmentCreate {
     my ( $Self, %Param ) = @_;
 
-    # check needed stuff
     for my $Needed (qw(CalendarID Title StartTime EndTime UserID)) {
         if ( !$Param{$Needed} ) {
             $Kernel::OM->Get('Kernel::System::Log')->Log(
@@ -339,8 +338,8 @@ sub AppointmentCreate {
     }
 
     push @Bind, \$Param{CalendarID}, \$Param{UniqueID}, \$Param{Title}, \$Param{Description},
-        \$Param{Location}, \$Param{StartTime},   \$Param{EndTime},   \$Param{AllDay},
-        \$Arrays{TeamID},  \$Arrays{ResourceID}, \$Param{Recurring}, \$Param{RecurrenceType},
+        \$Param{Location},             \$Param{StartTime},            \$Param{EndTime},   \$Param{AllDay},
+        \$Arrays{TeamID},              \$Arrays{ResourceID},          \$Param{Recurring}, \$Param{RecurrenceType},
         \$Arrays{RecurrenceFrequency}, \$Param{RecurrenceCount},      \$Param{RecurrenceInterval},
         \$Param{RecurrenceUntil},      \$Param{RecurrenceID},         \$Arrays{RecurrenceExclude},
         \$Param{NotificationDate},     \$Param{NotificationTemplate}, \$Param{NotificationCustom},
@@ -497,7 +496,7 @@ Result => 'HASH':
             NotificationCustomDateTime            => '2016-01-02 16:00:00',
             TicketAppointmentRuleID               => '9bb20ea035e7a9930652a9d82d00c725',    # for ticket appointments only!
         },
-        ...
+        # ...
     ];
 
 Result => 'ARRAY':
@@ -525,13 +524,13 @@ sub AppointmentList {
 
     # cache keys
     my $CacheType        = $Self->{CacheType} . 'List' . $Param{CalendarID};
-    my $CacheKeyTitle    = $Param{Title} || 'any';
+    my $CacheKeyTitle    = $Param{Title}       || 'any';
     my $CacheKeyDesc     = $Param{Description} || 'any';
-    my $CacheKeyLocation = $Param{Location} || 'any';
-    my $CacheKeyStart    = $Param{StartTime} || 'any';
-    my $CacheKeyEnd      = $Param{EndTime} || 'any';
-    my $CacheKeyTeam     = $Param{TeamID} || 'any';
-    my $CacheKeyResource = $Param{ResourceID} || 'any';
+    my $CacheKeyLocation = $Param{Location}    || 'any';
+    my $CacheKeyStart    = $Param{StartTime}   || 'any';
+    my $CacheKeyEnd      = $Param{EndTime}     || 'any';
+    my $CacheKeyTeam     = $Param{TeamID}      || 'any';
+    my $CacheKeyResource = $Param{ResourceID}  || 'any';
 
     if ( defined $Param{Title} && $Param{Title} =~ /^[\*]+$/ ) {
         $CacheKeyTitle = 'any';
@@ -747,7 +746,7 @@ sub AppointmentDays {
     # cache keys
     my $CacheType     = $Self->{CacheType} . 'Days' . $Param{UserID};
     my $CacheKeyStart = $Param{StartTime} || 'any';
-    my $CacheKeyEnd   = $Param{EndTime} || 'any';
+    my $CacheKeyEnd   = $Param{EndTime}   || 'any';
 
     # check time
     if ( $Param{StartTime} ) {
@@ -1086,6 +1085,110 @@ sub AppointmentGet {
     return %Result;
 }
 
+=head2 AppointmentRecurringGet()
+
+Get data of all of an appointment's recurring appointments.
+
+    my @RecurringAppointments = $AppointmentObject->AppointmentRecurringGet(
+        AppointmentID => 1,                                  # (required)
+                                                             # or
+        UniqueID      => '20160101T160000-71E386@localhost', # (required)
+        CalendarID    => 1,                                  # (required)
+    );
+
+Returns an array:
+
+    @RecurringAppointment = (
+        {
+            AppointmentID       => 2,
+            ParentID            => 1,
+            CalendarID          => 1,
+            UniqueID            => '20160101T160000-71E386@localhost',
+            Title               => 'Webinar',
+            Description         => 'How to use Process tickets...',
+            Location            => 'Straubing',
+            StartTime           => '2016-01-01 16:00:00',
+            EndTime             => '2016-01-01 17:00:00',
+            AllDay              => 0,
+            TeamID              => [ 1 ],
+            ResourceID          => [ 1, 3 ],
+            Recurring           => 1,
+            RecurrenceType      => 'Daily',
+            RecurrenceFrequency => 1,
+            RecurrenceCount     => 1,
+            RecurrenceInterval  => 2,
+            RecurrenceUntil     => '2016-01-10 00:00:00',
+            RecurrenceID        => '2016-01-10 00:00:00',
+            RecurrenceExclude   => [
+                '2016-01-10 00:00:00',
+                '2016-01-11 00:00:00',
+            ],
+            NotificationTime                  => '2016-01-01 17:0:00',
+            NotificationTemplate              => 'Custom',
+            NotificationCustomUnitCount       => '12',
+            NotificationCustomUnit            => 'minutes',
+            NotificationCustomUnitPointOfTime => 'beforestart',
+
+            TicketAppointmentRuleID => '9bb20ea035e7a9930652a9d82d00c725',  # for ticket appointments only!
+            CreateTime              => '2016-01-01 00:00:00',
+            CreateBy                => 2,
+            ChangeTime              => '2016-01-01 00:00:00',
+            ChangeBy                => 2,
+        },
+        # ...
+    );
+
+=cut
+
+sub AppointmentRecurringGet {
+    my ( $Self, %Param ) = @_;
+
+    my $DBObject = $Kernel::OM->Get('Kernel::System::DB');
+
+    # Fetch parent appointment.
+    my %Appointment = $Self->AppointmentGet(%Param);
+    return if !%Appointment;
+
+    my $AppointmentID = $Appointment{AppointmentID};
+
+    # Get IDs of recurring appointments.
+    my $SQL = '
+        SELECT id
+        FROM   calendar_appointment
+        WHERE  parent_id = ?
+    ';
+    my @Bind = (
+        \$AppointmentID,
+    );
+
+    return if !$DBObject->Prepare(
+        SQL  => $SQL,
+        Bind => \@Bind,
+    );
+
+    my %RecurringAppointmentIDs;
+    while ( my @Row = $DBObject->FetchrowArray() ) {
+        my $RecurringAppointmentID = $Row[0];
+        $RecurringAppointmentIDs{$RecurringAppointmentID} = 1;
+    }
+    return if !%RecurringAppointmentIDs;
+
+    my @RecurringAppointments;
+
+    RECURRINGAPPOINTMENTID:
+    for my $RecurringAppointmentID ( sort keys %RecurringAppointmentIDs ) {
+        my %RecurringAppointment = $Self->AppointmentGet(
+            AppointmentID => $RecurringAppointmentID,
+
+        );
+        next RECURRINGAPPOINTMENTID if !%RecurringAppointment;
+
+        push @RecurringAppointments, \%RecurringAppointment;
+    }
+
+    return @RecurringAppointments;
+}
+
 =head2 AppointmentUpdate()
 
 updates an existing appointment.
@@ -1351,11 +1454,11 @@ sub AppointmentUpdate {
     return if !$Kernel::OM->Get('Kernel::System::DB')->Do(
         SQL  => $SQL,
         Bind => [
-            \$Param{CalendarID}, \$Param{Title},   \$Param{Description}, \$Param{Location},
-            \$Param{StartTime},  \$Param{EndTime}, \$Param{AllDay},      \$Arrays{TeamID},
-            \$Arrays{ResourceID},          \$Param{Recurring},       \$Param{RecurrenceType},
-            \$Arrays{RecurrenceFrequency}, \$Param{RecurrenceCount}, \$Param{RecurrenceInterval},
-            \$Param{RecurrenceUntil}, \$RecurrenceExclude, \$Param{NotificationDate},
+            \$Param{CalendarID},                            \$Param{Title},   \$Param{Description}, \$Param{Location},
+            \$Param{StartTime},                             \$Param{EndTime}, \$Param{AllDay},      \$Arrays{TeamID},
+            \$Arrays{ResourceID},                           \$Param{Recurring},       \$Param{RecurrenceType},
+            \$Arrays{RecurrenceFrequency},                  \$Param{RecurrenceCount}, \$Param{RecurrenceInterval},
+            \$Param{RecurrenceUntil},                       \$RecurrenceExclude,      \$Param{NotificationDate},
             \$Param{NotificationTemplate},                  \$Param{NotificationCustom},
             \$Param{NotificationCustomRelativeUnitCount},   \$Param{NotificationCustomRelativeUnit},
             \$Param{NotificationCustomRelativePointOfTime}, \$Param{NotificationCustomDateTime},
@@ -1646,11 +1749,12 @@ sub GetUniqueID {
     $StartTimeStrg =~ s/[-:]//g;
     $StartTimeStrg =~ s/\s/T/;
 
-    # get system FQDN
-    my $FQDN = $Kernel::OM->Get('Kernel::Config')->Get('FQDN');
+    # get system ExternalFQDN
+    my $ConfigObject = $Kernel::OM->Get('Kernel::Config');
+    my $ExternalFQDN = $ConfigObject->Get('ExternalFQDN') || $ConfigObject->Get('FQDN');
 
     # return UniqueID
-    return "$StartTimeStrg-$Hash\@$FQDN";
+    return "$StartTimeStrg-$Hash\@$ExternalFQDN";
 }
 
 =head2 AppointmentUpcomingGet()
@@ -1910,7 +2014,7 @@ sub AppointmentFutureTasksUpdate {
     if ( !$TaskID ) {
         $Kernel::OM->Get('Kernel::System::Log')->Log(
             Priority => 'error',
-            Message =>
+            Message  =>
                 "Could not schedule future task for AppointmentID $UpcomingAppointments[0]->{AppointmentID}!",
         );
         return;
@@ -2501,6 +2605,19 @@ sub _CalculateRecurrenceTime {
     # We will modify this object throughout the function.
     my $CurrentTimeObject = $Param{CurrentTime};
 
+    # Use floating time zone to be able to add and subtract whole days, etc.
+    # regardless of daylight saving time.
+    #
+    # This might lead to other problems like calculating a time that does not exist,
+    # e.g. 2:00 AM on the day of the DST switch from winter to summer time in time zone
+    # Europe/Berlin (1:59 AM switches to 3:00 AM).
+    #
+    # Kernel::System::DateTime takes care of this by adjusting the date.
+    my $CurrentTimeObjectValues = $CurrentTimeObject->Get();
+    my $TimeZone                = $CurrentTimeObjectValues->{TimeZone};
+
+    $CurrentTimeObject->ToTimeZone( TimeZone => 'floating' );
+
     if ( $Param{Appointment}->{RecurrenceType} eq 'Daily' ) {
 
         # Add one day.
@@ -2706,6 +2823,8 @@ sub _CalculateRecurrenceTime {
         return;
     }
 
+    $CurrentTimeObject->ToTimeZone( TimeZone => $TimeZone );
+
     return $CurrentTimeObject;
 }
 
@@ -2720,6 +2839,7 @@ values (hour, minute and second) as in Time.
     );
 
 Returns:
+
     $Result = '2016-02-01 00:01:00';
 
 =cut
@@ -2760,6 +2880,7 @@ Returns how many calendar weeks has passed between two unix times.
     );
 
 returns:
+
     $CWDiff = 5;
 
 =cut
