@@ -929,6 +929,7 @@ sub CustomerCompanyUpdate {
     for my $Entry ( @{ $Self->{CustomerCompanyMap}->{Map} } ) {
         next FIELD if $Entry->[0] =~ /^UserPassword$/i;
         next FIELD if $Entry->[5] eq 'dynamic_field';     # skip dynamic fields
+        next FIELD if $Entry->[7] eq '1';                 # skip read-only fields
         push @Fields, $Entry->[2] . ' = ?';
         push @Values, \$Param{ $Entry->[0] };
     }
@@ -937,6 +938,9 @@ sub CustomerCompanyUpdate {
         push @Values, \$Param{UserID};
     }
 
+    # return if no fields need to be updated
+    return 1 if !@Fields;
+
     # create SQL statement
     my $SQL = "UPDATE $Self->{CustomerCompanyTable} SET ";
     $SQL .= join( ', ', @Fields );
