@@ -1,5 +1,5 @@
 # --
-# Copyright (C) 2021-2022 Znuny GmbH, https://znuny.org/
+# Copyright (C) 2021 Znuny GmbH, https://znuny.org/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
 # the enclosed file COPYING for license information (AGPL). If you
@@ -7,9 +7,7 @@
 # --
 
 # because of direct open() calls
-## nofilter(TidyAll::Plugin::OTRS::Perl::PerlCritic)
-
-## nofilter(TidyAll::Plugin::OTRS::Perl::Pod::SpellCheck)
+## nofilter(TidyAll::Plugin::Znuny::Perl::PerlCritic)
 
 package Kernel::System::DBCRUD::History;
 
@@ -31,6 +29,10 @@ our %ConfigBackup;
 our @ConfigBackupColumns
     = qw(Columns Name Identifier DatabaseTable DefaultSortBy DefaultOrderBy CacheType CacheTTL PreventHistory FunctionDataAdd FunctionDataUpdate FunctionDataDelete FunctionDataGet FunctionDataListGet FunctionDataSearch FunctionDataSearchValue);
 
+=head1 NAME
+
+Kernel::System::DBCRUD::History
+
 =head2 HistoryBackendSet()
 
 This function sets the information for the history DBCRUD module
@@ -600,7 +602,7 @@ sub _DataHistoryGet {
         );
     }
 
-    # set default because we dont write the initial value of data add to the history
+    # set default because we don't write the initial value of data add to the history
     # because of performance reasons
     DEFAULT:
     for my $Key ( sort keys %DataGet ) {
@@ -662,9 +664,11 @@ sub _DataHistoryGet {
 
     # trigger event
     $Self->EventHandler(
-        Event  => $Self->{Name} . 'HistoryGet',
-        Data   => \%Entry,
-        UserID => $Param{UserID} || 1,
+        ModuleName        => ref $Self,
+        UseHistoryBackend => 1,
+        Event             => 'DBCRUDHistoryGet',
+        Data              => \%Entry,
+        UserID            => $Param{UserID} || 1,
     );
 
     return %Entry;
@@ -780,7 +784,7 @@ sub CreateMissingUUIDHistoryDatabaseTableColumn {
 
     $LogObject->Log(
         Priority => 'error',
-        Message =>
+        Message  =>
             "Column $UUIDHistoryDatabaseTableColumnName could not be created in database table $HistoryDatabaseTable.",
     );
     return;
@@ -877,7 +881,7 @@ sub MigrateUUIDHistoryDatabaseTableColumn {
 
         $LogObject->Log(
             Priority => 'error',
-            Message =>
+            Message  =>
                 "Error: Unable to execute SQL: $SQL",
         );
         return;
