@@ -1,6 +1,6 @@
 # --
 # Copyright (C) 2001-2021 OTRS AG, https://otrs.com/
-# Copyright (C) 2021-2022 Znuny GmbH, https://znuny.org/
+# Copyright (C) 2021 Znuny GmbH, https://znuny.org/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
 # the enclosed file COPYING for license information (GPL). If you
@@ -8,10 +8,11 @@
 # --
 
 package Kernel::System::TemplateGenerator;
-## nofilter(TidyAll::Plugin::OTRS::Perl::LayoutObject)
+## nofilter(TidyAll::Plugin::Znuny::Perl::LayoutObject)
 
 use strict;
 use warnings;
+use utf8;
 
 use Kernel::Language;
 
@@ -19,12 +20,13 @@ use Kernel::System::VariableCheck qw(:all);
 
 our @ObjectDependencies = (
     'Kernel::Config',
+    'Kernel::Output::HTML::Layout',
     'Kernel::System::AutoResponse',
     'Kernel::System::CommunicationChannel',
     'Kernel::System::CustomerUser',
+    'Kernel::System::DateTime',
     'Kernel::System::DynamicField',
     'Kernel::System::DynamicField::Backend',
-    'Kernel::System::Encode',
     'Kernel::System::HTMLUtils',
     'Kernel::System::Log',
     'Kernel::System::Queue',
@@ -35,8 +37,6 @@ our @ObjectDependencies = (
     'Kernel::System::Ticket',
     'Kernel::System::Ticket::Article',
     'Kernel::System::User',
-    'Kernel::Output::HTML::Layout',
-    'Kernel::System::DateTime',
 );
 
 =head1 NAME
@@ -119,7 +119,8 @@ sub Salutation {
     if ( $Self->{RichText} && $Salutation{ContentType} =~ /text\/plain/i ) {
         $Salutation{ContentType} = 'text/html';
         $Salutation{Text}        = $Kernel::OM->Get('Kernel::System::HTMLUtils')->ToHTML(
-            String => $Salutation{Text},
+            String                 => $Salutation{Text},
+            DoNotReplaceHardBreaks => 1,
         );
     }
 
@@ -146,6 +147,7 @@ sub Salutation {
         TicketData => \%Ticket,
         Data       => $Param{Data},
         UserID     => $Param{UserID},
+        Recipient  => $Param{Recipient},
     );
 
     # add urls
@@ -228,7 +230,8 @@ sub Signature {
     if ( $Self->{RichText} && $Signature{ContentType} =~ /text\/plain/i ) {
         $Signature{ContentType} = 'text/html';
         $Signature{Text}        = $Kernel::OM->Get('Kernel::System::HTMLUtils')->ToHTML(
-            String => $Signature{Text},
+            String                 => $Signature{Text},
+            DoNotReplaceHardBreaks => 1,
         );
     }
 
@@ -273,8 +276,8 @@ sub Signature {
 generate sender address (FROM string) for emails
 
     my $Sender = $TemplateGeneratorObject->Sender(
-        QueueID    => 123,
-        UserID     => 123,
+        QueueID => 123,
+        UserID  => 123,
     );
 
 returns:
@@ -359,7 +362,7 @@ sub Sender {
 generate template
 
     my $Template = $TemplateGeneratorObject->Template(
-        TemplateID => 123
+        TemplateID => 123,
         TicketID   => 123,                  # Optional
         Data       => $ArticleHashRef,      # Optional
         UserID     => 123,
@@ -367,7 +370,7 @@ generate template
 
 Returns:
 
-    $Template =>  'Some text';
+    $Template => 'Some text';
 
 =cut
 
@@ -385,10 +388,16 @@ sub Template {
         }
     }
 
-    my %Template = $Kernel::OM->Get('Kernel::System::StandardTemplate')->StandardTemplateGet(
+    my $StandardTemplateObject = $Kernel::OM->Get('Kernel::System::StandardTemplate');
+
+    my %Template = $StandardTemplateObject->StandardTemplateGet(
         ID => $Param{TemplateID},
     );
 
+    # Convert comma separated TemplateType to array for better searching.
+    my @TemplateTypes = split( /\s*,\s*/, $Template{TemplateType} );
+    $Template{TemplateType} = \@TemplateTypes;
+
     # do text/plain to text/html convert
     if (
         $Self->{RichText}
@@ -398,7 +407,8 @@ sub Template {
     {
         $Template{ContentType} = 'text/html';
         $Template{Template}    = $Kernel::OM->Get('Kernel::System::HTMLUtils')->ToHTML(
-            String => $Template{Template},
+            String                 => $Template{Template},
+            DoNotReplaceHardBreaks => 1,
         );
     }
 
@@ -434,8 +444,9 @@ sub Template {
     }
 
     # If template type is 'Create' and there is customer user information, treat it as a ticket param in order to
-    # correctly replace customer user tags. See bug#14455.
-    if ( $Template{TemplateType} eq 'Create' && $Param{CustomerUserID} ) {
+    # correctly replace customer user tags.
+    my $IsCreateTemplate = grep { $_ eq 'Create' } @{ $Template{TemplateType} };
+    if ( $IsCreateTemplate && $Param{CustomerUserID} ) {
         $Ticket{CustomerUserID} = $Param{CustomerUserID};
     }
 
@@ -450,19 +461,20 @@ sub Template {
         Forward => 1,
         Note    => 1,
     );
+    my $IsSupportedType = grep { $SupportedTypes{$_} } @{ $Template{TemplateType} };
 
     my $TemplateText = $Template{Template} || '';
-    my $TemplateType = $Template{TemplateType};
+    my @TemplateType = @{ $Template{TemplateType} };
 
     # Remove unsupported tags only for some template types.
-    if ( !$SupportedTypes{ $Template{TemplateType} } ) {
+    if ( !$IsSupportedType ) {
         $TemplateText = $Self->_RemoveUnSupportedTag(
             Text                 => $Template{Template} || '',
             ListOfUnSupportedTag => \@ListOfUnSupportedTag,
         );
 
         # Reset template type for unsupported tag.
-        $TemplateType = '';
+        @TemplateType = ();
     }
 
     # replace place holder stuff
@@ -473,15 +485,15 @@ sub Template {
         Data       => $Param{Data} || {},
         UserID     => $Param{UserID},
         Language   => $Language,
-        Template   => $TemplateType,
+        Template   => \@TemplateType,
     );
 
     if ( $Self->{RichText} ) {
+        $TemplateText =~ s/&amp;nbsp;/&nbsp;/g;
         $TemplateText =~ s/&lt;/</g;
         $TemplateText =~ s/&gt;/>/g;
         $TemplateText =~ s/&quot;/"/g;
         $TemplateText =~ s/&apos;/'/g;
-        $TemplateText =~ s/&nbsp;/ /g;
     }
 
     return $TemplateText;
@@ -492,10 +504,10 @@ sub Template {
 generate internal or external notes
 
     my $GenericAgentArticle = $TemplateGeneratorObject->GenericAgentArticle(
-        Notification    => $NotificationDataHashRef,
-        TicketID        => 123,
-        UserID          => 123,
-        Data            => $ArticleHashRef,             # Optional
+        Notification => $NotificationDataHashRef,
+        TicketID     => 123,
+        UserID       => 123,
+        Data         => $ArticleHashRef,             # Optional
     );
 
 =cut
@@ -534,7 +546,8 @@ sub GenericAgentArticle {
     {
         $Template{ContentType} = 'text/html';
         $Template{Body}        = $Kernel::OM->Get('Kernel::System::HTMLUtils')->ToHTML(
-            String => $Template{Body},
+            String                 => $Template{Body},
+            DoNotReplaceHardBreaks => 1,
         );
     }
 
@@ -593,7 +606,7 @@ generate attributes
     my %Attributes = $TemplateGeneratorObject->Attributes(
         TicketID   => 123,
         ArticleID  => 123,
-        ResponseID => 123
+        ResponseID => 123,
         UserID     => 123,
         Action     => 'Forward', # Possible values are Reply and Forward, Reply is default.
     );
@@ -632,7 +645,7 @@ sub Attributes {
     $Param{Data}->{Subject} = $TicketObject->TicketSubjectBuild(
         TicketNumber => $Ticket{TicketNumber},
         Subject      => $Param{Data}->{Subject} || '',
-        Action       => $Param{Action} || '',
+        Action       => $Param{Action}          || '',
     );
 
     # get sender address
@@ -648,28 +661,12 @@ sub Attributes {
 
 generate response
 
-AutoResponse
-    TicketID
-        Owner
-        Responsible
-        CUSTOMER_DATA
-    ArticleID
-        CUSTOMER_SUBJECT
-        CUSTOMER_EMAIL
-    UserID
-
-    To
-    Cc
-    Bcc
-    Subject
-    Body
-    ContentType
-
     my %AutoResponse = $TemplateGeneratorObject->AutoResponse(
         TicketID         => 123,
         OrigHeader       => {},
         AutoResponseType => 'auto reply',
         UserID           => 123,
+        UserType         => 'Agent', # optional
     );
 
 =cut
@@ -771,7 +768,8 @@ sub AutoResponse {
     if ( $Self->{RichText} && $AutoResponse{ContentType} =~ /text\/plain/i ) {
         $AutoResponse{ContentType} = 'text/html';
         $AutoResponse{Text}        = $Kernel::OM->Get('Kernel::System::HTMLUtils')->ToHTML(
-            String => $AutoResponse{Text},
+            String                 => $AutoResponse{Text},
+            DoNotReplaceHardBreaks => 1,
         );
     }
 
@@ -852,8 +850,9 @@ sub AutoResponse {
         );
 
         $AutoResponse{Text} = $Kernel::OM->Get('Kernel::System::HTMLUtils')->DocumentComplete(
-            Charset => 'utf-8',
-            String  => $AutoResponse{Text},
+            Charset  => 'utf-8',
+            String   => $AutoResponse{Text},
+            UserType => $Param{UserType} || 'Agent',
         );
     }
 
@@ -1089,7 +1088,8 @@ sub NotificationEvent {
     if ( $Self->{RichText} && $Notification{ContentType} =~ /text\/plain/i ) {
         $Notification{ContentType} = 'text/html';
         $Notification{Body}        = $Kernel::OM->Get('Kernel::System::HTMLUtils')->ToHTML(
-            String => $Notification{Body},
+            String                 => $Notification{Body},
+            DoNotReplaceHardBreaks => 1,
         );
     }
 
@@ -1176,6 +1176,8 @@ sub _Replace {
         }
     }
 
+    my $HTMLUtilsObject = $Kernel::OM->Get('Kernel::System::HTMLUtils');
+
     # check for mailto links
     # since the subject and body of those mailto links are
     # uri escaped we have to uri unescape them, replace
@@ -1438,8 +1440,9 @@ sub _Replace {
             ATTRIBUTE:
             for my $Attribute ( sort keys %Recipient ) {
                 next ATTRIBUTE if !$Recipient{$Attribute};
-                $Recipient{$Attribute} = $Kernel::OM->Get('Kernel::System::HTMLUtils')->ToHTML(
-                    String => $Recipient{$Attribute},
+                $Recipient{$Attribute} = $HTMLUtilsObject->ToHTML(
+                    String                 => $Recipient{$Attribute},
+                    DoNotReplaceHardBreaks => 1,
                 );
             }
         }
@@ -1469,8 +1472,9 @@ sub _Replace {
             ATTRIBUTE:
             for my $Attribute ( sort keys %Owner ) {
                 next ATTRIBUTE if !$Owner{$Attribute};
-                $Owner{$Attribute} = $Kernel::OM->Get('Kernel::System::HTMLUtils')->ToHTML(
-                    String => $Owner{$Attribute},
+                $Owner{$Attribute} = $HTMLUtilsObject->ToHTML(
+                    String                 => $Owner{$Attribute},
+                    DoNotReplaceHardBreaks => 1,
                 );
             }
         }
@@ -1500,8 +1504,9 @@ sub _Replace {
             ATTRIBUTE:
             for my $Attribute ( sort keys %Responsible ) {
                 next ATTRIBUTE if !$Responsible{$Attribute};
-                $Responsible{$Attribute} = $Kernel::OM->Get('Kernel::System::HTMLUtils')->ToHTML(
-                    String => $Responsible{$Attribute},
+                $Responsible{$Attribute} = $HTMLUtilsObject->ToHTML(
+                    String                 => $Responsible{$Attribute},
+                    DoNotReplaceHardBreaks => 1,
                 );
             }
         }
@@ -1526,8 +1531,9 @@ sub _Replace {
         ATTRIBUTE:
         for my $Attribute ( sort keys %CurrentUser ) {
             next ATTRIBUTE if !$CurrentUser{$Attribute};
-            $CurrentUser{$Attribute} = $Kernel::OM->Get('Kernel::System::HTMLUtils')->ToHTML(
-                String => $CurrentUser{$Attribute},
+            $CurrentUser{$Attribute} = $HTMLUtilsObject->ToHTML(
+                String                 => $CurrentUser{$Attribute},
+                DoNotReplaceHardBreaks => 1,
             );
         }
     }
@@ -1550,8 +1556,9 @@ sub _Replace {
         ATTRIBUTE:
         for my $Attribute ( sort keys %Ticket ) {
             next ATTRIBUTE if !$Ticket{$Attribute};
-            $Ticket{$Attribute} = $Kernel::OM->Get('Kernel::System::HTMLUtils')->ToHTML(
-                String => $Ticket{$Attribute},
+            $Ticket{$Attribute} = $HTMLUtilsObject->ToHTML(
+                String                 => $Ticket{$Attribute},
+                DoNotReplaceHardBreaks => 1,
             );
         }
     }
@@ -1676,6 +1683,17 @@ sub _Replace {
     # replace it
     $HashGlobalReplace->( $Tag, %Ticket, %DynamicFieldDisplayValues );
 
+    # OTRS_TICKET_LAST_ARTICLE_ID
+    my $ArticleObject = $Kernel::OM->Get('Kernel::System::Ticket::Article');
+    if ( $Ticket{TicketID} ) {
+        my @LastTicketArticle = $ArticleObject->ArticleList(
+            TicketID => $Ticket{TicketID},
+            OnlyLast => 1,
+        );
+        my $LastTicketArticleID = @LastTicketArticle ? $LastTicketArticle[0]->{ArticleID} : '';
+        $Param{Text} =~ s{$Start OTRS_TICKET_LAST_ARTICLE_ID $End}{$LastTicketArticleID}gixms;
+    }
+
     # COMPAT
     $Param{Text} =~ s/$Start OTRS_TICKET_ID $End/$Ticket{TicketID}/gixms;
     $Param{Text} =~ s/$Start OTRS_TICKET_NUMBER $End/$Ticket{TicketNumber}/gixms;
@@ -1704,9 +1722,12 @@ sub _Replace {
     # - if ArticleID is sent, data is from selected article.
     # - if ArticleID is not sent, data is from last customer/agent/any article.
     if ( $Param{Template} && $Ticket{TicketID} ) {
-        my $ArticleObject = $Kernel::OM->Get('Kernel::System::Ticket::Article');
 
-        if ( $Param{Template} eq 'Note' ) {
+        my $IsNoteTemplate    = grep { $_ eq 'Note' } @{ $Param{Template} };
+        my $IsAnswerTemplate  = grep { $_ eq 'Answer' } @{ $Param{Template} };
+        my $IsForwardTemplate = grep { $_ eq 'Forward' } @{ $Param{Template} };
+
+        if ($IsNoteTemplate) {
 
             # Get last article from agent.
             my @AgentArticles = $ArticleObject->ArticleList(
@@ -1740,7 +1761,7 @@ sub _Replace {
                 $Param{Data}->{Body}    = $CustomerArticle{Body};
             }
         }
-        elsif ( $Param{Template} eq 'Answer' || $Param{Template} eq 'Forward' ) {
+        elsif ( $IsAnswerTemplate || $IsForwardTemplate ) {
 
             # If $Param{Data} has agent article data, we will set subject and body in $Param{DataAgent}
             # to values from $Param{Data} in order to right replacing of OTRS_AGENT_SUBJECT/BODY tags.
@@ -1768,8 +1789,9 @@ sub _Replace {
             for my $Attribute ( sort keys %Data ) {
                 next ATTRIBUTE if !$Data{$Attribute};
 
-                $Data{$Attribute} = $Kernel::OM->Get('Kernel::System::HTMLUtils')->ToHTML(
-                    String => $Data{$Attribute},
+                $Data{$Attribute} = $HTMLUtilsObject->ToHTML(
+                    String                 => $Data{$Attribute},
+                    DoNotReplaceHardBreaks => 1,
                 );
             }
         }
@@ -1829,9 +1851,16 @@ sub _Replace {
                         $NewOldBody =~ s/(<br\/>)\s{0,20}$//gs;
                     }
 
+                    # replace hard breaks with paragraphs before there will
+                    # be added any "HTML" content like blockquote that would
+                    # make it complex to replace hard breaks afterwards
+                    $NewOldBody = $HTMLUtilsObject->ToHTMLReplaceWithParagraphs(
+                        String => $NewOldBody,
+                    );
+
                     # add quote
                     $NewOldBody = "<blockquote type=\"cite\">$NewOldBody</blockquote>";
-                    $NewOldBody = $Kernel::OM->Get('Kernel::System::HTMLUtils')->DocumentCleanup(
+                    $NewOldBody = $HTMLUtilsObject->DocumentCleanup(
                         String => $NewOldBody,
                     );
                 }
@@ -1960,8 +1989,9 @@ sub _Replace {
             ATTRIBUTE:
             for my $Attribute ( sort keys %CustomerUser ) {
                 next ATTRIBUTE if !$CustomerUser{$Attribute};
-                $CustomerUser{$Attribute} = $Kernel::OM->Get('Kernel::System::HTMLUtils')->ToHTML(
-                    String => $CustomerUser{$Attribute},
+                $CustomerUser{$Attribute} = $HTMLUtilsObject->ToHTML(
+                    String                 => $CustomerUser{$Attribute},
+                    DoNotReplaceHardBreaks => 1,
                 );
             }
         }
@@ -1977,6 +2007,10 @@ sub _Replace {
     $Tag = $Start . 'OTRS_AGENT_';
     $Param{Text} =~ s/$Tag.+?$End/-/gi;
 
+    $Param{Text} = $HTMLUtilsObject->ToHTMLReplaceWithParagraphs(
+        String => $Param{Text},
+    ) if ( $Param{RichText} );
+
     return $Param{Text};
 }
 
