@@ -34,6 +34,8 @@ our @ObjectDependencies = (
     'Kernel::System::Ticket',
     'Kernel::System::Ticket::Article',
     'Kernel::System::User',
+    'Kernel::System::Valid',
+    'Kernel::System::Mention',
 );
 
 sub new {
@@ -252,6 +254,8 @@ sub Run {
                 next TRANSPORT;
             }
 
+            my @ValidIDs = $Kernel::OM->Get('Kernel::System::Valid')->ValidIDsGet();
+
             # check if transport is usable
             next TRANSPORT if !$TransportObject->IsUsable();
 
@@ -301,6 +305,17 @@ sub Run {
                     next BUNDLE if !$Bundle->{Recipient}->{UserID};
                 }
 
+                # Check if customer user is invalid.
+                if (
+                    $Bundle->{Recipient}->{Source}
+                    && $Bundle->{Recipient}->{Source} eq 'CustomerUser'
+                    && $Bundle->{Recipient}->{ValidID}
+                    && !grep { $Bundle->{Recipient}->{ValidID} eq $_ } @ValidIDs
+                    )
+                {
+                    next BUNDLE;
+                }
+
                 my $Success = $Self->_SendRecipientNotification(
                     TicketID              => $Param{Data}->{TicketID},
                     Notification          => $Bundle->{Notification},
@@ -373,39 +388,24 @@ sub _NotificationFilter {
 
     my $ArticleObject = $Kernel::OM->Get('Kernel::System::Ticket::Article');
 
+    # get not ticket related attributes
+    my $IgnoredAttributes
+        = $Kernel::OM->Get('Kernel::Config')->Get('Ticket::Event::NotificationEvent::IgnoredAttributes') || {};
+    my %IgnoredAttributesHash;
+    for my $Key ( sort keys %{$IgnoredAttributes} ) {
+        if ( IsHashRefWithData( $IgnoredAttributes->{$Key} ) ) {
+            %IgnoredAttributesHash = ( %IgnoredAttributesHash, %{ $IgnoredAttributes->{$Key} } );
+        }
+    }
+
     # get the search article fields to retrieve values for
     my %ArticleSearchableFields = $ArticleObject->ArticleSearchableFieldsList();
 
     KEY:
     for my $Key ( sort keys %{ $Notification{Data} } ) {
 
-        # TODO: This function here should be fixed to not use hardcoded attribute values!
         # ignore not ticket related attributes
-        next KEY if $Key eq 'Recipients';
-        next KEY if $Key eq 'SkipRecipients';
-        next KEY if $Key eq 'RecipientAgents';
-        next KEY if $Key eq 'RecipientGroups';
-        next KEY if $Key eq 'RecipientRoles';
-        next KEY if $Key eq 'TransportEmailTemplate';
-        next KEY if $Key eq 'Events';
-        next KEY if $Key eq 'ArticleSenderTypeID';
-        next KEY if $Key eq 'ArticleIsVisibleForCustomer';
-        next KEY if $Key eq 'ArticleCommunicationChannelID';
-        next KEY if $Key eq 'ArticleAttachmentInclude';
-        next KEY if $Key eq 'IsVisibleForCustomer';
-        next KEY if $Key eq 'Transports';
-        next KEY if $Key eq 'OncePerDay';
-        next KEY if $Key eq 'VisibleForAgent';
-        next KEY if $Key eq 'VisibleForAgentTooltip';
-        next KEY if $Key eq 'LanguageID';
-        next KEY if $Key eq 'SendOnOutOfOffice';
-        next KEY if $Key eq 'AgentEnabledByDefault';
-        next KEY if $Key eq 'EmailSecuritySettings';
-        next KEY if $Key eq 'EmailSigningCrypting';
-        next KEY if $Key eq 'EmailMissingCryptingKeys';
-        next KEY if $Key eq 'EmailMissingSigningKeys';
-        next KEY if $Key eq 'EmailDefaultSigningKeys';
-        next KEY if $Key eq 'NotificationType';
+        next KEY if %IgnoredAttributesHash && $IgnoredAttributesHash{$Key};
 
         # ignore article searchable fields
         next KEY if $ArticleSearchableFields{$Key};
@@ -599,6 +599,7 @@ sub _RecipientsGet {
         my $CheckItemObject     = $Kernel::OM->Get('Kernel::System::CheckItem');
         my $SystemAddressObject = $Kernel::OM->Get('Kernel::System::SystemAddress');
         my $UserObject          = $Kernel::OM->Get('Kernel::System::User');
+        my $MentionObject       = $Kernel::OM->Get('Kernel::System::Mention');
 
         RECIPIENT:
         for my $Recipient ( @{ $Notification{Data}->{Recipients} } ) {
@@ -725,7 +726,7 @@ sub _RecipientsGet {
                 }
             }
 
-            # Other OTRS packages might add other kind of recipients that are normally handled by
+            # Other packages might add other kind of recipients that are normally handled by
             #   other modules then an elsif condition here is useful.
             elsif ( $Recipient eq 'Customer' ) {
 
@@ -968,6 +969,15 @@ sub _RecipientsGet {
 
                 push @RecipientUsers, \%Recipient;
             }
+            elsif ( $Recipient eq 'AllMentionedUsers' ) {
+                my $TicketMentions = $MentionObject->GetTicketMentions(
+                    TicketID => $Param{Data}->{TicketID},
+                ) // [];
+
+                for my $TicketMention ( @{$TicketMentions} ) {
+                    push @{ $Notification{Data}->{RecipientAgents} }, $TicketMention->{UserID};
+                }
+            }
         }
     }
 
@@ -1169,16 +1179,16 @@ sub _SendRecipientNotification {
                 $_->{HistoryType} eq 'SendCustomerNotification'
                     && $_->{Name} eq
                     "\%\%$Param{Recipient}->{UserEmail}"
-            }
-            reverse @HistoryLines;
+                }
+                reverse @HistoryLines;
         }
         else {
             $LastNotificationHistory = first {
                 $_->{HistoryType} eq 'SendAgentNotification'
                     && $_->{Name} eq
                     "\%\%$Param{Notification}->{Name}\%\%$Param{Recipient}->{UserLogin}\%\%$Param{Transport}"
-            }
-            reverse @HistoryLines;
+                }
+                reverse @HistoryLines;
         }
 
         if ( $LastNotificationHistory && $LastNotificationHistory->{CreateTime} ) {
